//***************************************************************************
// Group Linslot / Linux - Slotrace Manager
// File linslot.hpp
// Date 05.12.06 - Jrg Wendel
// This code is distributed under the terms and conditions of the
// GNU GENERAL PUBLIC LICENSE. See the file COPYING for details.
//***************************************************************************

#ifndef _LINSLOT_H_
#define _LINSLOT_H_

//***************************************************************************
// Includes
//***************************************************************************

#include <QMainWindow>
#include <QThread>
#include <QReadWriteLock>
#include <QTimer>
#include <QDir>

#include <ui_linslot.h>

#include <highscore.hpp>
#include <setup.hpp>
#include <common.hpp>
#include <iothread.hpp>
#include <sqlite.hpp>

//***************************************************************************
// Class LinslotWindow
//***************************************************************************

class LinslotWindow : public QMainWindow, private Ui::LinslotWindow, public SlotService
{
      Q_OBJECT

   public:

      // declarations

      enum Misc
      {
         sizeName       = 50,
         bitsPerByte    = 8,

         bounceTime     = 300000,    // Seconds (0.3 sec)
         bounceTimeSlot = 1000000,   // Seconds (1.0 sec)
         slotCount      = 2,

         // the arduino analog io (0-5)

         bitGhostInSlot1   = 0,   // analog in for ghost car slot 1
         bitGhostInSlot3   = 1,   // analog in for ghost car slot 3
         bitAnalogInput0   = 2,   // free
         bitAnalogInput1   = 3,   // free
         bitAnalogInput2   = 4,   // free
         bitAnalogInput3   = 5    // free
      };

      enum InputSignal
      {
         bitStartKey,         // external start/stop key

         bitPanicKey1,        // panic keys
         bitPanicKey2,
         bitPanicKey3,
         bitPanicKey4,

         bitIrSlot1,          // slot signal
         bitIrSlot2,
         bitIrSlot3,
         bitIrSlot4,

         bitFuelStartSlot1,   // fueling start signal
         bitFuelStartSlot2,
         bitFuelStartSlot3,
         bitFuelStartSlot4,

         bitFuelEndSlot1,     // fueling end signal
         bitFuelEndSlot2,
         bitFuelEndSlot3,
         bitFuelEndSlot4,

         bitInputCount
      };

      enum OutputSignal
      {
         bitLightRed1,
         bitLightRed2,
         bitLightRed3,
         bitLightRed4,
         bitLightRed5,
         bitLightGreen,

         bitLightYellow1,
         bitLightYellow2,
         bitLightYellow3,
         bitLightYellow4,

         bitPowerSlot1,
         bitPowerSlot2,
         bitPowerSlot3,
         bitPowerSlot4,

         bitPmwOutSlot1,     // out for Ghostcar Slot 1
         bitPmwOutSlot2,     // out for Ghostcar Slot 3

         bitOutputCount
      };

      enum PowerState
      {
         psSlot1 = 1,
         psSlot2 = 2,
         psSlot3 = 4,
         psSlot4 = 8,

         psAll = psSlot1 | psSlot2 | psSlot3 | psSlot4
      };

      enum LedState
      {
         lsAllOff     = 0,

         lsGreen      = 1,
         lsRed1       = 2,
         lsRed2       = 4,
         lsRed3       = 8,
         lsRed4       = 16,
         lsRed5       = 32,
         lsYellow1    = 64,
         lsYellow2    = 128,

         lsPhase1     = lsRed1,
         lsPhase2     = lsPhase1 | lsRed2,
         lsPhase3     = lsPhase2 | lsRed3,
         lsPhase4     = lsPhase3 | lsRed4,
         lsPhase5     = lsPhase4 | lsRed5,
         lsPhase6     = lsGreen,
         
         lsSlotPower1 = lsYellow1,
         lsSlotPower2 = lsYellow2,
         lsSlotPower  = lsSlotPower1 | lsSlotPower2,

         lsAllOn      = lsPhase5 | lsGreen | lsSlotPower
      };

      enum TriggerEdge
      {
         teRising,
         teFalling
      };
      
      enum OutputMode
      {
         omNormal,
         omPwm
      };

      struct InputDefinition
      {
         int bit;
         TriggerEdge mode;
      };

      struct OutputDefinition
      {
         int bit;
         OutputMode mode;
      };

      struct Slot
      {
         timeval lastSignal;
         int round;
         char driver[sizeName+TB];
         int penalty;
         double fuelLevel;            // fuel tank level
         int fueling;
         QTimer* fuelTimer;
      };

      // object

      LinslotWindow();
      ~LinslotWindow();

      void init();
      void exit();

   protected:

      // functions

      void atSlotSignal(int slot, const timeval* tp);
      void atFuelSignal(int slot, const timeval* tp, int fuelStartSignal);
      void atStartCountdown();
      void atStart();
      void atFinish(int slot);
      void atAbort(const char* info);
      void atJumpTheGun(int slot);
      void atNoFuel(int slot);
      void atStop(const char* info);

      int getImagesFor(int led, const char* &ledOn, const char* &ledOff);
      void setLeds(int mask);
      void switchLeds(int mask, int state);

      void setSlotPower(int slot, int flag);
      unsigned int getChanges(IoEvent &ioEvent);
      int isActiveEdge(int bit, int value);
      int inputBitOf(int function);
      int outputBitOf(int function);

      void resetWidgets();
      void applyOptions();
      void playSound(QString file);
      void atFuelTimer(int slot);

      // db stuff

      int openDb();
      int createDb();
      int saveRace();
      int showDb();
      int getDriverId(const char* driver);
      int getCourseId();

      // data

      int testMode;
      IoThread* thread;
      QTimer* timer;
      QTimer* timerElapsed;
      QTimer* timerPenalty;
      Slot theSlots[slotCount];
      timeval raceStart;
      int raceRunning;
      int countdownStarted;
      int countdown;
      QObject* ledsParent;
      double fastLapTime;
      int withSound;

      SqliteDb* db;

      char* resourcePath;
      QString configPath;

      // dialogs
      
      SetupDialog* setupDialog;
      HighscoreDialog* highscoreDialog;

      static InputDefinition inputBits[bitInputCount];
      static OutputDefinition outputBits[bitOutputCount];

   private slots:

      void onIoSignal(const IoEvent ioEvent);
      void on_pushButtonPower_clicked();
      void on_pushButtonSaveToDb_clicked();
      void on_pushButtonHallOfFame_clicked();
      void on_pushButtonStartRace_clicked();
      void on_pushButtonOptions_clicked();
      void on_pushButtonInitUsb_clicked();
      void on_toolButtonTest_clicked();
      void onTimer();
      void onFuelTimerSlot1();
      void onFuelTimerSlot2();
      void onElapsedTimer();
      void onPenaltyTimer();
      void onOptionsAccepted();
      void on_comboBoxDriver1_currentIndexChanged(QString value);
      void on_comboBoxDriver2_currentIndexChanged(QString value);
};

//***************************************************************************
#endif // _LINSLOT_H_
